@@ -429,6 +429,28 @@ cached_dir_remove_subdir (CachedDir  *dir,
   return FALSE;
 }
 
+static void
+cached_dir_invoke_monitors (CachedDir *dir)
+{
+  GSList *tmp;
+
+  tmp = dir->monitors;
+  while (tmp != NULL)
+    {
+      CachedDirMonitor *monitor = tmp->data;
+      GSList           *next    = tmp->next;
+
+      monitor->callback (monitor->ed, monitor->user_data);
+
+      tmp = next;
+    }
+
+  if (dir->parent)
+    {
+      cached_dir_invoke_monitors (dir->parent);
+    }
+}
+
 static void
 handle_cached_dir_changed (GnomeVFSMonitorHandle    *handle,
                            const char               *monitor_uri,
@@ -504,18 +526,7 @@ handle_cached_dir_changed (GnomeVFSMonitorHandle    *handle,
 
   if (handled)
     {
-      GSList *tmp;
-
-      tmp = dir->monitors;
-      while (tmp != NULL)
-        {
-          CachedDirMonitor *monitor = tmp->data;
-	  GSList           *next    = tmp->next;
-
-          monitor->callback (monitor->ed, monitor->user_data);
-
-          tmp = next;
-        }
+      cached_dir_invoke_monitors (dir);
     }
 }
 
@@ -1332,7 +1343,7 @@ get_inverse_func (EntryDirectory  *ed,
 
   /* if not in the original set, add to inverse set */
 
-  if (desktop_entry_set_lookup (set, relative_path) != NULL)
+  if (desktop_entry_set_lookup (set, file_id ? file_id : relative_path) != NULL)
     return TRUE;
 
   if (ed->is_legacy && !desktop_entry_has_categories (entry))
